/*=========================================================================

Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.

This software is distributed WITHOUT ANY WARRANTY; without even 
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the above copyright notices for more information.

                ----------------------------------------

              Add On files for VirtoolsMP development team

Author  : Sylvain Benner
Support : benner@virtools.com

Desc : This generator provides additional infos in order to write the 
       specific parameters needed by MP visual projects.
		 It creates a local generator from the VirtoolsMP addon package.

=========================================================================*/
#include "windows.h" // this must be first to define GetCurrentDirectory
#include "cmGlobalVisualStudio71GeneratorMP.h"
#include "cmLocalVisualStudio7GeneratorMP.h"
#include "cmGeneratedFileStream.h"
#include "cmSourceFile.h"
#include "cmTarget.h"
#include "cmMakefile.h"
#include "cmake.h"


cmGlobalVisualStudio71GeneratorMP::cmGlobalVisualStudio71GeneratorMP()
{
	this->FindMakeProgramFile = "CMakeVS71FindMake.cmake";
	this->ProjectConfigurationSectionName = "ProjectConfiguration";
}
//----------------------------------------------------------------------------
///! Create a local generator appropriate to this Global Generator
cmLocalGenerator *cmGlobalVisualStudio71GeneratorMP::CreateLocalGenerator()
{
	// retrieve the name of the platform
	cmLocalVisualStudio7GeneratorMP *lg = new cmLocalVisualStudio7GeneratorMP;
	lg->SetVersion71();
	lg->SetGlobalGenerator(this);

	return lg;
}
//----------------------------------------------------------------------------
void cmGlobalVisualStudio71GeneratorMP::GenerateConfigurations(cmMakefile* mf)
{
	if (this->CMakeInstance->GetCacheDefinition("MP_CONFIGURATIONS_MODE"))
	{
		std::string sConfigurations = this->GetCMakeInstance()->GetCacheDefinition("CMAKE_CONFIGURATION_TYPES");
		this->Configurations.clear();
		cmsys::SystemTools::Split(sConfigurations.c_str(), this->Configurations, ';');
	}
	else
	{
		mf->AddCacheDefinition(
			"CMAKE_CONFIGURATION_TYPES",
			"Debug;Release;MinSizeRel;RelWithDebInfo",
			"Semicolon separated list of supported configuration types, "
			"only supports Debug, Release, MinSizeRel, and RelWithDebInfo, "
			"anything else will be ignored.",
			cmCacheManager::STRING);
		cmGlobalVisualStudio7Generator::GenerateConfigurations(mf);
	}
}
//----------------------------------------------------------------------------
// Write a SLN file to the stream
void cmGlobalVisualStudio71GeneratorMP::WriteSLNFile(std::ostream& fout,
																	cmLocalGenerator* root,
																	std::vector<cmLocalGenerator*>& generators)
{
	// GUID Pool
	const char* pGUIDPool = this->LocalGenerators[0]->GetMakefile()->GetDefinition("GUID_POOL");
	if(pGUIDPool)
	{
		this->m_GUIDPool.SetPoolPath(pGUIDPool);
	}

	// Write out the header for a SLN file
	this->WriteSLNHeader(fout);

	// Get the start directory with the trailing slash
	std::string rootdir = root->GetMakefile()->GetStartOutputDirectory();
	rootdir += "/";
	bool doneAllBuild = true;
	bool doneCheckBuild = false;
	bool doneRunTests = false;
	bool doneInstall  = true;
	bool doneEditCache = false;
	bool doneRebuildCache = false;
	bool donePackage = false;

	// For each cmMakefile, create a VCProj for it, and
	// add it to this SLN file
	unsigned int i;
	for(i = 0; i < generators.size(); ++i)
	{
		if(this->IsExcluded(root, generators[i]))
		{
			continue;
		}
		cmMakefile* mf = generators[i]->GetMakefile();

		// Get the source directory from the makefile
		std::string dir = mf->GetStartOutputDirectory();
		// remove the home directory and / from the source directory
		// this gives a relative path 
		cmSystemTools::ReplaceString(dir, rootdir.c_str(), "");

		// Get the list of create dsp files names from the cmVCProjWriter, more
		// than one dsp could have been created per input CMakeLists.txt file
		// for each target
		std::vector<std::string> dspnames = 
			static_cast<cmLocalVisualStudio7GeneratorMP *>(generators[i])
			->GetCreatedProjectNames();
		cmTargets &tgts = generators[i]->GetMakefile()->GetTargets();
		cmTargets::iterator l = tgts.begin();
		for(std::vector<std::string>::iterator si = dspnames.begin(); 
			l != tgts.end() && si != dspnames.end(); ++l)
		{
			// special handling for the current makefile
			if(mf == generators[0]->GetMakefile())
			{
				dir = "."; // no subdirectory for project generated
				// if this is the special ALL_BUILD utility, then
				// make it depend on every other non UTILITY project.
				// This is done by adding the names to the GetUtilities
				// vector on the makefile
				if(l->first == "ALL_BUILD" && !doneAllBuild)
				{
					unsigned int j;
					for(j = 0; j < generators.size(); ++j)
					{
						cmTargets &atgts = generators[j]->GetMakefile()->GetTargets();
						for(cmTargets::iterator al = atgts.begin();
							al != atgts.end(); ++al)
						{
							if (al->second.IsInAll())
							{
								if (al->second.GetType() == cmTarget::UTILITY ||
									al->second.GetType() == cmTarget::GLOBAL_TARGET)
								{
									l->second.AddUtility(al->first.c_str());
								}
								else
								{
									l->second.AddLinkLibrary(al->first,cmTarget::GENERAL);
								}
							}
						}
					}
				}
			}
			// Write the project into the SLN file
			if (strncmp(l->first.c_str(), "INCLUDE_EXTERNAL_MSPROJECT", 26) == 0)
			{
				cmCustomCommand cc = l->second.GetPostBuildCommands()[0];
				const cmCustomCommandLines& cmds = cc.GetCommandLines();
				std::string project = cmds[0][0];
				std::string location = cmds[0][1];
				this->WriteExternalProject(fout, project.c_str(), location.c_str(), cc.GetDepends());
			}
			else 
			{
				if ((l->second.GetType() != cmTarget::INSTALL_FILES)
					&& (l->second.GetType() != cmTarget::INSTALL_PROGRAMS))
				{
					bool skip = false;
					if(l->first == "ALL_BUILD" )
					{
						if(doneAllBuild)
						{
							skip = true;
						}
						else
						{
							doneAllBuild = true;
						}
					}
					if(l->first == CMAKE_CHECK_BUILD_SYSTEM_TARGET)
					{
						if(doneCheckBuild)
						{
							skip = true;
						}
						else
						{
							doneCheckBuild = true;
						}
					}
					if(l->first == "INSTALL")
					{
						//if(doneInstall)
						//{
							skip = true;
						//}
						//else
						//{
						//	doneInstall = true;
						//}
					}
					if(l->first == "RUN_TESTS")
					{
						if(doneRunTests)
						{
							skip = true;
						}
						else
						{
							doneRunTests = true;
						}
					}
					if(l->first == "EDIT_CACHE")
					{
						if(doneEditCache)
						{
							skip = true;
						}
						else
						{
							doneEditCache = true;
						}
					}
					if(l->first == "REBUILD_CACHE")
					{
						if(doneRebuildCache)
						{
							skip = true;
						}
						else
						{
							doneRebuildCache = true;
						}
					}
					if(l->first == "PACKAGE")
					{
						if(donePackage)
						{
							skip = true;
						}
						else
						{
							donePackage = true;
						}
					}
					if(!skip)
					{
						this->WriteProject(fout, si->c_str(), dir.c_str(),l->second);
					}
					++si;
				}
			}
		}
	}

	// Write solution group declaration
	this->WriteSolutionGroups(fout);

	fout << "Global\n";
	this->WriteSolutionConfigurations(fout);
	fout << "\tGlobalSection(" << this->ProjectConfigurationSectionName
		<< ") = postSolution\n";
	// loop over again and compute the depends
	for(i = 0; i < generators.size(); ++i)
	{
		cmMakefile* mf = generators[i]->GetMakefile();
		cmLocalVisualStudio7GeneratorMP* pg =  
			static_cast<cmLocalVisualStudio7GeneratorMP*>(generators[i]);
		// Get the list of create dsp files names from the cmVCProjWriter, more
		// than one dsp could have been created per input CMakeLists.txt file
		// for each target
		std::vector<std::string> dspnames = 
			pg->GetCreatedProjectNames();
		cmTargets &tgts = pg->GetMakefile()->GetTargets();
		cmTargets::iterator l = tgts.begin();
		std::string dir = mf->GetStartDirectory();
		for(std::vector<std::string>::iterator si = dspnames.begin(); 
			l != tgts.end() && si != dspnames.end(); ++l)
		{
			if (strncmp(l->first.c_str(), "INCLUDE_EXTERNAL_MSPROJECT", 26) == 0)
			{
				cmCustomCommand cc = l->second.GetPostBuildCommands()[0];
				const cmCustomCommandLines& cmds = cc.GetCommandLines();
				std::string project = cmds[0][0];
				this->WriteProjectConfigurations(fout, project.c_str(), l->second.IsInAll());
			}
			else if ((l->second.GetType() != cmTarget::INSTALL_FILES)
				&& (l->second.GetType() != cmTarget::INSTALL_PROGRAMS))
			{
				if(*si == "INSTALL")
				{
					// skip the install project
					++si;
					continue;
				}
				else if(l->second.GetProperty("IS_MP_TARGET"))
				{
					// target added with ADD_TARGET_MP command
					this->WriteProjectConfigurations(fout, si->c_str(), l->second.IsInAll(), l->second);
				}
				else
				{
					this->WriteProjectConfigurations(fout, si->c_str(), l->second.IsInAll());
				}
				++si;
			}
		}
	}
	fout << "\tEndGlobalSection\n";

	// Write the solution folder dependencies
	this->WriteSolutionGroupDependencies(fout);

	// Write the footer for the SLN file
	this->WriteSLNFooter(fout);
}
//----------------------------------------------------------------------------
void cmGlobalVisualStudio71GeneratorMP::OutputSLNFile(
	cmLocalGenerator* root,
   std::vector<cmLocalGenerator*>& generators)
{
  if(generators.size() == 0)
    {
    return;
    }

  if(root->GetNoUpdate())
  {
	  // do not update this file
		return;
  }

  // sln file
  this->CurrentProject = root->GetMakefile()->GetProjectName();
  std::string fname = root->GetMakefile()->GetStartOutputDirectory();
  fname += "/";
  fname += root->GetMakefile()->GetProjectName();
  std::string correspondingProject = fname + ".vcproj";
  const char* pForceGeneration = root->GetMakefile()->GetDefinition("FORCE_SOLUTION_GENERATION");
  if(!pForceGeneration && cmSystemTools::FileExists(correspondingProject.c_str()))
  {
	   // do not produce this sln if there is a corresponding vcproj wit the same name
		return;
  }
  fname += ".sln";
  // save the name of the real dsp file
  std::string realSLN = fname;
  fname += ".cmake";
  std::ofstream fout(fname.c_str());
  if(!fout)
    {
    cmSystemTools::Error("Error Writing ", fname.c_str());
    }
  this->WriteSLNFile(fout, root, generators);
  fout.close();

  if(cmSystemTools::FilesDiffer(fname.c_str(), realSLN.c_str()))
  {
    // create a file to warn about the newest file
    std::string warnFile;
    warnFile = root->GetMakefile()->GetStartOutputDirectory();
    warnFile += "/";
    warnFile += root->GetMakefile()->GetProjectName();
    warnFile += "_to_update.cmake";
    fout.open(warnFile.c_str());
    fout << fname.c_str();
    fout << "\n";
    fout << realSLN.c_str();
    fout.close();
  }
  // if the dsp file has changed, then write it.
  cmSystemTools::CopyFileIfDifferent(fname.c_str(), realSLN.c_str());

/*
  // write the vspscc file
  fname = root->GetMakefile()->GetStartOutputDirectory(); 
  fname += "/";
  fname += "ALL_BUILD";
  fname += ".vcproj.vspscc";
  // save the name of the real dsp file
  std::string realVSPSCC = fname;

  // check if the file already exists (if so, we don't need to regenerate it)
  if (cmSystemTools::FileExists(fname.c_str()) == false)
  {
		fname += ".cmake"; 
		fout.open(fname.c_str());
		if(!fout)
			{
			cmSystemTools::Error("Error Writing ", fname.c_str());
			}
		this->WriteVSPSCCFile(fout);
		fout.close();
		// if the dsp file has changed, then write it.
		cmSystemTools::CopyFileIfDifferent(fname.c_str(), realVSPSCC.c_str());
  }
*/
}
//----------------------------------------------------------------------------
std::string cmGlobalVisualStudio71GeneratorMP::GetGUIDMP(const char* name, cmTarget* target)
{
  std::string guidStoreName = name;
  guidStoreName += "_GUID_CMAKE";
  const char* storedGUID = 
    this->CMakeInstance->GetCacheDefinition(guidStoreName.c_str());
  if(storedGUID)
    {
    return std::string(storedGUID);

  }
  if(target && target->GetMakefile()->IsOn("CHECK_FOR_DEPENDENCIES"))
  {
     cmSystemTools::Error("Unknown Target referenced : ", name);
  }
  return "";
}
//----------------------------------------------------------------------------
void cmGlobalVisualStudio71GeneratorMP::WriteVSPSCCFile(std::ostream& fout)
{
	fout << "\"\"\n";
	fout << "{\n";
	fout << "\"FILE_VERSION\" = \"9237\"\n";
	fout << "\"ENLISTMENT_CHOICE\" = \"NEVER\"\n";
	fout << "\"PROJECT_FILE_RELATIVE_PATH\" = \"\"\n";
	fout << "\"NUMBER_OF_EXCLUDED_FILES\" = \"0\"\n";
	fout << "\"ORIGINAL_PROJECT_FILE_PATH\" = \"\"\n";
	fout << "\"NUMBER_OF_NESTED_PROJECTS\" = \"0\"\n";
	fout << "\"SOURCE_CONTROL_SETTINGS_PROVIDER\" = \"PROJECT\"\n";
	fout << "}\n";
}
//----------------------------------------------------------------------------
void
cmGlobalVisualStudio71GeneratorMP::WriteSolutionConfigurations(std::ostream& fout)
{
	fout << "\tGlobalSection(SolutionConfiguration) = preSolution\n";
	for(std::vector<std::string>::iterator i = this->Configurations.begin();
		i != this->Configurations.end(); ++i)
	{
		fout << "\t\t" << *i << " = " << *i << "\n";
	}
	fout << "\tEndGlobalSection\n";
}
//----------------------------------------------------------------------------
// Write a dsp file into the SLN file,
// Note, that dependencies from executables to 
// the libraries it uses are also done here
void
cmGlobalVisualStudio71GeneratorMP::WriteProject(std::ostream& fout,
															 const char* dspname,
															 const char* dir,
															 cmTarget& t)
{
	std::string d;
	if(cmSystemTools::FileIsFullPath(dir))
	{
		d = cmSystemTools::RelativePath(this->GetLocalGenerator(0)->GetMakefile()->GetStartOutputDirectory(), dir);
		if(d.empty())
		{
			d = ".";
		}
	}
	else
	{
		d = dir;
	}
	cmSystemTools::ReplaceString(d, "/", "\\");
	fout << "Project(\"{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}\") = \"" 
		<< dspname << "\", \""
		<< d << "\\" << dspname << ".vcproj\", \"{"
		<< this->GetGUID(dspname) << "}\"\n";
	fout << "\tProjectSection(ProjectDependencies) = postProject\n";
	this->WriteProjectDepends(fout, dspname, dir, t);
	fout << "\tEndProjectSection\n";

	fout <<"EndProject\n";
}
//----------------------------------------------------------------------------
// Write a dsp file into the SLN file,
// Note, that dependencies from executables to 
// the libraries it uses are also done here
void
cmGlobalVisualStudio71GeneratorMP::WriteProjectDepends(std::ostream& fout,
							 const char* dspname,
							 const char*, cmTarget& target)
{
	// insert Begin Project Dependency  Project_Dep_Name project stuff here 
	if (target.GetType() != cmTarget::STATIC_LIBRARY)
	{
		cmTarget::LinkLibraryVectorType::const_iterator j, jend;
		j = target.GetLinkLibraries().begin();
		jend = target.GetLinkLibraries().end();
		for(;j!= jend; ++j)
		{
			if(j->first != dspname)
			{
				// is the library part of this SLN ? If so add dependency
				if(this->FindTarget(this->CurrentProject.c_str(), j->first.c_str()))
				{
					fout << "\t\t{" << this->GetGUID(j->first.c_str()) << "} = {"
						<< this->GetGUID(j->first.c_str()) << "}\n";
				}
			}
		}
	}

	std::set<cmStdString>::const_iterator i, end;
	// write utility dependencies.
	i = target.GetUtilities().begin();
	end = target.GetUtilities().end();
	for(;i!= end; ++i)
	{
		if(*i != dspname)
		{
			std::string name = i->c_str();
			if(strncmp(name.c_str(), "INCLUDE_EXTERNAL_MSPROJECT", 26) == 0)
			{
				// kind of weird removing the first 27 letters.
				// my recommendatsions:
				// use cmCustomCommand::GetCommand() to get the project name
				// or get rid of the target name starting with "INCLUDE_EXTERNAL_MSPROJECT_" and use another 
				// indicator/flag somewhere.  These external project names shouldn't conflict with cmake 
				// target names anyways.
				name.erase(name.begin(), name.begin() + 27);
			}

		   std::string guid = this->GetGUIDMP(name.c_str(), &target);
			if(guid.size() == 0)
			{
				if(target.GetMakefile()->IsOn("CHECK_FOR_DEPENDENCIES"))
				{ 
					std::string m = "Target: ";
					m += target.GetName();
					m += " depends on unknown target: ";
					m += name;
					cmSystemTools::Error(m.c_str());
				}
			}
			else
			{
				fout << "\t\t{" << guid << "} = {" << guid << "}\n";
			}
		}
	}
}
//----------------------------------------------------------------------------
// Write a dsp file into the SLN file,
// Note, that dependencies from executables to 
// the libraries it uses are also done here
void cmGlobalVisualStudio71GeneratorMP::WriteExternalProject(std::ostream& fout, 
																			  const char* name,
																			  const char* location,
																			  const std::vector<std::string>& depends)
{ 
	//*** BEGIN EMERGENT MODIFICATION
	//*** support for external project files that are not C++

	// Create an uppercase version of location
	char* location_uppercase = NULL;
	{
		size_t location_strlen = strlen(location);
		location_uppercase = new char[location_strlen+1];
		strcpy(location_uppercase, location);
		for (char* c = location_uppercase; *c != NULL; c++)
		{
			*c = toupper(*c);
		}
	}

	// Check what type of project it is, to know what type of GUID to use.
	const char* project_type_guid = "8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942"; //C
	if (strstr(location_uppercase, ".CSPROJ"))
		project_type_guid = "FAE04EC0-301F-11D3-BF4B-00C04F79EFBC";
	if (strstr(location_uppercase, ".VBPROJ"))
		project_type_guid = "F184B08F-C81C-45F6-A57F-5ABD9991F28F";
	delete [] location_uppercase;
	location_uppercase = NULL;

	std::string d = cmSystemTools::ConvertToOutputPath(location);
	fout << "Project(\"{" << project_type_guid << "}\") = \""
		<< name << "\", \""
		<< d << "\", \"{"
		<< this->GetGUID(name)
		<< "}\"\n";

	//*** END EMERGENT MODIFICATION 

	// write out the dependencies here
	// VS 7.1 includes dependencies with the project instead of in the global section
	if(!depends.empty())
	{
		fout << "\tProjectSection(ProjectDependencies) = postProject\n";
		std::vector<std::string>::const_iterator it;
		for(it = depends.begin(); it != depends.end(); ++it)
		{
			if(it->size() > 0)
			{
				fout << "\t\t{" 
					<< this->GetGUID(it->c_str()) 
					<< "} = {" 
					<< this->GetGUID(it->c_str()) 
					<< "}\n";
			}
		}
		fout << "\tEndProjectSection\n";
	}  

	fout << "EndProject\n";


}
//----------------------------------------------------------------------------
// Write a dsp file into the SLN file,
// Note, that dependencies from executables to 
// the libraries it uses are also done here
void 
cmGlobalVisualStudio71GeneratorMP::WriteProjectConfigurations(std::ostream& fout, 
															  const char* name, 
															  bool in_all_build)
{
	std::string guid = this->GetGUID(name);
	for(std::vector<std::string>::iterator i = this->Configurations.begin();
		i != this->Configurations.end(); ++i)
	{
		fout << "\t\t{" << guid << "}." << *i << ".ActiveCfg = " << *i << "|" << this->PlatformName << "\n";
		if (in_all_build)
		{
			fout << "\t\t{" << guid << "}." << *i << ".Build.0 = " << *i << "|" << this->PlatformName << "\n";
		}
	}
}
//----------------------------------------------------------------------------
// Write a dsp file into the SLN file,
// Note, that dependencies from executables to 
// the libraries it uses are also done here
void 
cmGlobalVisualStudio71GeneratorMP::WriteProjectConfigurations(

	std::ostream& fout,
	const char* name,
	bool in_all_build,
	cmTarget& iTarget

	)
{
	bool uses_config = false;
	std::string guid = this->GetGUID(name);

	for(std::vector<std::string>::iterator i = this->Configurations.begin();
		i != this->Configurations.end(); ++i)
	{
		bool uses_config = false;
		std::string sConfig(*i);
		std::string sReplacementConfig(sConfig);
		int nPos = sConfig.find("Custom");
		if(nPos != sConfig.npos)
		{
			// Custom configuration, check if the target uses this configuration
			if(iTarget.GetProperty(sConfig.c_str()) != 0)
			{
				// use the custom configuration
				std::string sProperty(iTarget.GetProperty(sConfig.c_str()));
				uses_config = (sProperty.size() != 0);
			}
			else
			{
				// do not use the custom configuration
				// Get the replacement configuration
				nPos = sConfig.find_last_of("_");
				sReplacementConfig = sConfig.substr(nPos);
			}
		}
		else if(in_all_build)
		{
			uses_config = true;
		}

		// Active config
		fout << "\t\t{" << guid << "}." << sConfig << ".ActiveCfg = " << sReplacementConfig << "|" << this->PlatformName << "\n";
		// Build config
		if(uses_config)
		{
			fout << "\t\t{" << guid << "}." << sConfig << ".Build.0 = " << sConfig << "|" << this->PlatformName << "\n";
		}

	}
}

// === SOLUTION_GROUP ===

void cmGlobalVisualStudio71GeneratorMP::WriteSolutionGroup(

	std::ostream&  fout,
	cmSourceGroup* sg,
	std::string*   sParent_GUID

)
{
	// Here you can generate a new GUID
	std::string sGUIDName(sg->GetShortName());
	sGUIDName += "_";
	sGUIDName += *sParent_GUID;
	this->CreateGUID(sGUIDName.c_str());
	std::string sGUID(this->GetGUID(sGUIDName.c_str()));

	// Write the solution group declaration
	fout << "Project(\"{2150E333-8FDC-42A3-9474-1A3956D46DE8}\") = \"";
	fout << sg->GetShortName() << "\", \"" << sg->GetShortName() << "\", {" << sGUID << "}\n";
	fout << "EndProject\n";

	// Map this solutions group to its parent solution group
	if(sParent_GUID)
	{
		this->SolutionGroupMap[sGUID] = *sParent_GUID;
	}

	// Loop through every projects contained in this folder and map them to their solution group owner 
	const std::vector<const cmSourceFile *> &cmakelistsFiles = sg->GetSourceFiles();
	for(std::vector<const cmSourceFile *>::const_iterator sf =
	    cmakelistsFiles.begin(); sf != cmakelistsFiles.end(); ++sf)
	{
		this->SolutionGroupMap[this->GetGUID((*sf)->GetSourceName().c_str(), 0)] = sGUID;
	}

	// Loop through every solution groups children and map them to their solution group parent.
	std::vector<cmSourceGroup> children  = sg->GetGroupChildren();
	for(unsigned int i=0;i<children.size();++i)
	{
		cmSourceGroup& child = children[i];
		this->WriteSolutionGroup(fout, &child, &sGUID);
	}
}
//----------------------------------------------------------------------------
void cmGlobalVisualStudio71GeneratorMP::WriteSolutionGroups(std::ostream& fout)
{
	// Loop through every solution group.
	std::vector<cmSourceGroup> solutionGroups = this->GetLocalGenerator(0)->GetMakefile()->GetSolutionGroups();
	for(unsigned int i = 0; i < solutionGroups.size(); ++i)
	{
		cmSourceGroup& sg = solutionGroups[i];
		this->WriteSolutionGroup(fout, &sg, 0);
	}
}
//----------------------------------------------------------------------------
void cmGlobalVisualStudio71GeneratorMP::WriteSolutionGroupDependencies(std::ostream& fout)
{
	fout << "\tGlobalSection(NestedProjects) = preSolution\n";
	for(std::map<cmStdString, cmStdString>::iterator m = this->SolutionGroupMap.begin();
	    m != this->SolutionGroupMap.end(); ++m)
	{
		fout << "\t{" << m->first << "} = {" << m->second << "}\n";
	}
	fout << "\tEndGlobalSection\n";
}

// === end of SOLUTION_GROUP ===


// Standard end of dsw file
void cmGlobalVisualStudio71GeneratorMP::WriteSLNFooter(std::ostream& fout)
{
	fout << "\tGlobalSection(ExtensibilityGlobals) = postSolution\n"
		<< "\tEndGlobalSection\n"
		<< "\tGlobalSection(ExtensibilityAddIns) = postSolution\n"
		<< "\tEndGlobalSection\n"
		<< "EndGlobal\n";
}
//----------------------------------------------------------------------------
// ouput standard header for dsw file
void cmGlobalVisualStudio71GeneratorMP::WriteSLNHeader(std::ostream& fout)
{
	fout << "Microsoft Visual Studio Solution File, Format Version 8.00\n";
}
//----------------------------------------------------------------------------
void cmGlobalVisualStudio71GeneratorMP::GetDocumentation(cmDocumentationEntry& entry) const
{
	entry.name = this->GetName();
	entry.brief = "Generates Visual Studio .NET 2003 project files for VirtoolsMP.";
	entry.full = "";
}
