/*=========================================================================

Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.

This software is distributed WITHOUT ANY WARRANTY; without even 
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the above copyright notices for more information.

                ----------------------------------------

               Add-On file for VirtoolsMP development team

Author  : Sylvain Benner
Support : benner@virtools.com

Desc : This generator provides additional infos in order to write the 
       specific parameters needed by MP visual projects.

=========================================================================*/
#ifndef cmGlobalVisualStudio71GeneratorMP_h
#define cmGlobalVisualStudio71GeneratorMP_h

#include "cmGlobalVisualStudio71Generator.h"
#include "cmGUIDPoolMP.h"

class cmSourceGroup;

class cmGlobalVisualStudio71GeneratorMP : public cmGlobalVisualStudio71Generator
{
public:
	cmGlobalVisualStudio71GeneratorMP();
	static cmGlobalGenerator* New() 
	{ return new cmGlobalVisualStudio71GeneratorMP; }

	///! Get the name for the generator.
	virtual const char* GetName() const {
		return cmGlobalVisualStudio71GeneratorMP::GetActualName();}
	static const char* GetActualName() {return "Visual Studio 7 .NET 2003 for VirtoolsMP";}

	/** Get the documentation entry for this generator.  */
	virtual void GetDocumentation(cmDocumentationEntry& entry) const;

	///! Create a local generator appropriate to this Global Generator
	virtual cmLocalGenerator *CreateLocalGenerator();

	// platform name
   virtual void SetPlatformName(const char* iName) {this->PlatformName = iName;}
   virtual void SetRealPlatformName(const char* iName) {this->RealPlatformName = iName;}
	const std::string& GetCurrentProject() const {return this->CurrentProject;}

	// Create a GUID or get an existing one.
   virtual std::string GetGUIDMP(const char* name, cmTarget* target);


protected:
	void WriteVSPSCCFile(std::ostream& fout);
    virtual void OutputSLNFile(cmLocalGenerator* root, 
       std::vector<cmLocalGenerator*>& generators);	
	virtual void WriteSLNFile(std::ostream& fout, 
		cmLocalGenerator* root,
		std::vector<cmLocalGenerator*>& generators);
	virtual void WriteSolutionConfigurations(std::ostream& fout);
	virtual void WriteProject(std::ostream& fout, const char* name, const char* path, cmTarget &t);
	virtual void WriteProjectDepends(std::ostream& fout, 
		const char* name, const char* path, cmTarget &t);
	virtual void WriteProjectConfigurations(std::ostream& fout, const char* name,	bool in_all);
	virtual void WriteProjectConfigurations(std::ostream& fout, const char* name,	bool in_all, cmTarget& iTarget);
	virtual void WriteExternalProject(std::ostream& fout, const char* name,
		const char* path,
		const std::vector<std::string>& depends);
	virtual void WriteSLNFooter(std::ostream& fout);
	virtual void WriteSLNHeader(std::ostream& fout);
   virtual void GenerateConfigurations(cmMakefile* mf);

	// === SOLUTION_GROUP ===
		void WriteSolutionGroups(std::ostream& fout);
		void WriteSolutionGroup(std::ostream& fout, cmSourceGroup* sg, std::string* sParent_GUID);
		void WriteSolutionGroupDependencies(std::ostream& fout);
		std::map<cmStdString, cmStdString> SolutionGroupMap;
	// === end of SOLUTION_GROUP ===

	std::string ProjectConfigurationSectionName;
   std::string PlatformName; // Win32 or x64 
   std::string RealPlatformName; // Win32 or x64 

	cmGUIDPoolMP m_GUIDPool;
};
#endif
