/*=========================================================================

  Program:   CMake - Cross-Platform Makefile Generator
  Module:    $RCSfile: cmSourceGroupCommand.h,v $
  Language:  C++
  Date:      $Date: 2006/05/12 17:53:21 $
  Version:   $Revision: 1.14 $

  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

                ----------------------------------------

               Add-On file for VirtoolsMP development team

Author  : Sylvain Benner
Support : benner@virtools.com

=========================================================================*/
#ifndef cmSolutionGroupCommand_h
#define cmSolutionGroupCommand_h

#include "cmCommandMP.h"

/** \class cmSolutionGroupCommand
 * \brief Adds a cmSourceGroup to the cmMakefile.
 *
 * cmSolutionGroupCommand is used to define cmSourceGroups which split up
 * source files in to named, organized groups in the generated makefiles.
 */
class cmSolutionGroupCommand : public cmCommandMP
{
public:
  /**
   * This is a virtual constructor for the command.
   */
  virtual cmCommand* Clone() 
    {
    return new cmSolutionGroupCommand;
    }

  /**
   * This is called when the command is first encountered in
   * the CMakeLists.txt file.
   */
  virtual bool InitialPass(std::vector<std::string> const& args);
  
  /**
   * The name of the command as specified in CMakeList.txt.
   */
  virtual const char* GetName() {return "SOLUTION_GROUP_MP";}

  /**
   * Succinct documentation.
   */
  virtual const char* GetTerseDocumentation() 
    {
		 return "No documentation yet. Support: benner@virtools.com";
    }
  
  /**
   * More documentation.
   */
  virtual const char* GetFullDocumentation()
    {
       return "No documentation yet. Support: benner@virtools.com";
    }
   
  cmTypeMacro(cmSolutionGroupCommand, cmCommandMP);
};



#endif
